=begin

    Copyright The Resistance (C) 1995-2011

    Author: blass
    blass[at]sdf.lonestar.org

    Badass 0.1 alpha

    This file is part of Badass.

    Badass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Badass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Badass.  If not, see <http://www.gnu.org/licenses/>.

=end

module Racket
  module L4
    # Virtual Router Redundancy Protocol (VRRP)
    # http://tools.ietf.org/html/rfc2338
    # http://tools.ietf.org/html/rfc3768
    class VRRP < RacketPart
      # Version
      unsigned :version, 4
      # VRRP packet type 
      unsigned :type, 4 
      # Virtual Router Identifier (VRID)
      unsigned :id, 8
      # the sending VRRP router's priority for the virtual router.
      # Higher values equal higher priority.
      unsigned :priority, 8
      # Total number of IPs contained in this VRRP message
      unsigned :num_ips, 8
      # Authentication type (0, 1, 2)
      unsigned :auth_type, 8
      # Advertisement interval
      unsigned :interval, 8
      # Checksum
      unsigned :checksum, 16
      rest :payload
      
      # Add a new IP to this message
      def add_ip(ip)
        @ips << L3::Misc.ipv42long(ip)
      end
      
      # Add authentication data
      def add_auth(authdata)
        @authdata = authdata[0,8].ljust(32, "\x00")
      end
      
      # Validate the checksum
      def checksum?
        self.checksum == compute_checksum
      end
      
      # compute and set the checksum
      def checksum!
        self.checksum = compute_checksum
      end
      
      # (really, just set the checksum)
      def fix!
        self.payload = [@ips, @authdata].flatten.pack("N#{@ips.size}a*")
        self.num_ips = @ips.size
        self.checksum!
      end
      
      def initialize(*args)
        @ips = []
        @authdata = ""
        super
      end
      
      private
      def compute_checksum
        # pseudo header used for checksum calculation as per RFC 768 
        pseudo = [ ((self.version << 4) | self.type), self.id, self.priority, self.num_ips, self.auth_type, self.interval, 0, self.payload ] 
        L3::Misc.checksum(pseudo.pack("CCCCCCna*"))
      end
    end
  end
end
# vim: set ts=2 et sw=2:
